@echo off
rem
rem CFMASTER - Use CFMAIN{_CFbitmode} (currently v9.46.10 if 16-bit, or
rem            v9.51.02 if 32-bit) to launch the Crysfire system,
rem            including targeted indexing runs...
rem
rem R.Shirley, 23Jul04
rem
rem Supports:  Selecting 16- or 32-bit version, depending on %_CFbitmode%
rem            Inheriting/determining CFhome, to run under Win/NT and XP
rem            Performing direct-indexing runs via -Drun.
rem
rem ** If this fails to run, reporting "out of environment space", you have
rem    probably already used up most of your environment allocation, and so
rem    need to increase this to e.g. 1024 or 2048 bytes, using the SHELL
rem    command in CONFIG.SYS (/e parameter - e.g. /e:2048 for 2048 bytes)
rem
rem    Thus to provide 2048 bytes of environment space, insert the one of the
rem    following "shell=" lines in your CONFIG.SYS file, as appropriate:
rem
rem    (Windows 9x)       shell=C:\WINDOWS\COMMAND.COM C:\WINDOWS /e:2048 /p
rem    (DOS/Windows 3.x)  shell=C:\COMMAND.COM C:\DOS /e:2048 /p
rem
rem    Special action is required under Windows ME (which ignores CONFIG.SYS):
rem    Right-click on the Crysfire (or MSDOS) icon on the Windows desktop,
rem    and choose "Properties" then "Memory".  Change the setting for the
rem    "Initial Environment" from "Auto" to a suitable value such as 2048
rem    (the largest available is 4096)
rem
rem    Windows/XP and other NT-family O/S seem to manage without adjustment
rem

if (%1) == (/?) goto usage
if (%1) == (/h) goto usage
if (%1) == (-?) goto usage
if (%1) == (-h) goto usage
if (%1) == (?) goto usage
if (%1) == (h) goto usage
rem First delete any existing _SETPARM.BAT file
if exist _SETPARM.BAT del _SETPARM.BAT

rem _CFhome is now set by Crysfire, etc., before CFmaster is called
if not (%_CFhome%) == () goto set_Csrt
rem _CFhome hasn't been set, so let's try to do it now
call CF_home
if not (%_CFhome%) == () goto set_Csrt
echo ** Crysfire was unable to establish its home directory from CF_home.bat
echo (Please check that there's a valid copy of CF_home.bat on the execution path)
echo.
goto exit

:set_Csrt
rem For robustness, specify CSORT rather than test for a sort program
set _Csrt=%_CFhome%\CSORT -r
set _Csrtst=Yes
goto CFstart

rem Establish which sort program to use (currently deactivated)
echo.
echo Checking that there's a working SORT program on the execution PATH
echo.
set _Csrt=
if exist _Csrtb del _Csrtb
echo abc > _Csrta
rem Try MS SORT
SORT /R <_Csrta >_Csrtb
if exist _Csrtb set _Csrt=SORT /R
rem Also need to make a _Csrtst var, since Win/XP doesn't allow
rem spaces in the test strings used for "if ... = ..." tests
if exist _Csrtb set _Csrtst=Yes
if not (%_Csrtst%) == () goto CFstart
rem Nope, so try CSORT
rem CSORT parameter must be "-r" and not "-R", or usage will also be displayed
CSORT -r <_Csrta >_Csrtb
if exist _Csrtb set _Csrt=CSORT -r
if exist _Csrtb set _Csrtst=Yes
rem if not (%_Csrtst%) == () goto CFstart
if exist _Csrta del _Csrta
if exist _Csrtb del _Csrtb
echo  ** Warning **  Crysfire has not found a working SORT program on this system.
echo.
echo  If you proceed, you will still be able to run Crysfire commands and launch
echo  indexing runs, but Crysfire will be unable to display any sorted summary files
echo  or run the LePage program to produce a revised summary of the reduced cells
echo.
echo  To exit, press ENTER, then type QU (and ENTER) at the Crysfire command prompt
echo.
echo  This shouldn't be able to happen if Crysfire has been installed correctly,
echo  since the Crysfire 2002 distribution includes a backup sort utility CSORT.
echo  Check that CSORT.EXE has been copied correctly to your Crysfire directory,
echo  and that the Crysfire directory has been included in your execution PATH
echo.
pause

:CFstart
if exist _Csrta del _Csrta
if exist _Csrtb del _Csrtb

:SetIndexEdit
if not (%INDEXEDIT%) == () goto IndexEditSet
rem Try to sense whether we're running on W32, and if so set default output
rem viewer to WordPad (i.e. "Write") instead of MS EDIT.  (The lower scripts
rem Q2{id} & RUN{id} also do this, in case they're run directly)
if not (%windir%) == () set INDEXEDIT=Write
if %COMSPEC% == C:\WINDOWS\COMMAND.COM set INDEXEDIT=Write
rem Win/XP lacks winbootdir and has its own lengthy version of COMSPEC...
if not (%SystemRoot%) == () set INDEXEDIT=Write
:IndexEditSet

if (%_CFbitmode%) == (16) %_CFhome%\CFMAIN%_CFbitmode%
if (%_CFbitmode%) == (32) %_CFhome%\CFMAIN

rem exit if no parameter file was written
if not exist _SETPARM.BAT goto exit
rem Set environmental variables via BAT file written by CFMAIN%_CFbitmode%,
rem since TP5.5 seems not to support writing to the environment (only reading)
call _SETPARM
del _SETPARM.BAT

rem Skip to direct-run section if flagfile _CFdrun is present
if exist _CFdrun goto direct_run

rem From v9.33b (23Jun99), use one parameterised launcher based on QDAT input
rem filenames targeted via _C2Qid, instead of separate ones for each target
if (%_C2Qid%) == () goto no_qTarg
if (%_C2Qid%) == (MM) goto call_qQ2id
if not exist %_C2Qdset%.Q%_C2Qid% goto no_qData

:call_qQ2id
rem Use suffix %_Q2id_ver% for Q2id script names, to ensure matching versions
call %_CFhome%\Q2%_C2Qid%%_Q2id_ver% %_C2Qdset%
echo Indexing-target script finished
goto CFrestart

:no_qTarg
echo No indexing target requested for Q2{indx}
goto CFrestart
:no_qData
echo Can't find the targeted QDAT input data file %_C2Qdset%.Q%_C2Qid%
goto CFrestart

:direct_run
rem From v9.45.02 (30Jun02), replace the set of old RUN{indx} scripts by a
rem parallel parameterised Q2{indx} launcher, cued by the presence of a
rem _CFdrun flagfile to skip to direct-run native-format input datafiles,
rem targeted via _C2Qid
if (%_C2Qid%) == () goto no_dTarg
if (%_C2Qid%) == (HM) echo ** Direct-indexing-run requests aren't applicable to Hmap **
if (%_C2Qid%) == (HM) goto CFrestart
if (%_C2Qid%) == (MM) echo ** Direct-indexing-run requests aren't applicable to Hmap **
if (%_C2Qid%) == (MM) goto CFrestart
if not exist %_C2Qdset%.%_C2Qid%D goto no_dData

:call_dQ2id
rem Use suffix %_Q2id_ver% for Q2id script names, to ensure matching versions
call %_CFhome%\Q2%_C2Qid%%_Q2id_ver% %_C2Qdset% -Run
echo Direct (native input formats) indexing-target script finished
goto CFrestart

:no_dTarg
echo No direct-run indexing target requested
goto CFrestart
:no_dData
echo Can't find the targeted native %_C2Qid% input data file %_C2Qdset%.%_C2Qid%D
goto CFrestart

:CFrestart
if exist _CFdrun del _CFdrun >nul
echo.
echo The Crysfire front-end will now restart.  It will be a fresh instance and
echo so may not reproduce every changed setting that may have been made during
echo the run.  However, the current dataset will be automatically reloaded
echo (including its 2Theta zero correction)
echo.
echo If you don't want to continue running Crysfire, type QU (=QUit) at its
echo command prompt, to exit back to the operating system (Windows/DOS)
pause
goto CFstart

:usage
cls
rem This usage summary is for CRYSFIRE.BAT, the request having been passed down
rem to CFMAST{_CFbitmode} to execute
rem
echo CRYSFIRE System - A powder-indexing wizard constructed from scripts and
echo                   programs by Robin Shirley, using CFMAIN16 or CFMAIN32
echo                   as a front-end, depending on the bit-mode selected.
echo.
echo No parameters are needed (except ? or h to see this help information).
echo.
echo 1)  Type CRYSFIRE, then load your data (via the LO command)
echo.
echo 2)  Optionally carry out data-enhancement using self-calibration (SC), etc.
echo     Then launch an indexing run (IN) specifying the supported indexing
echo     program of your choice - advice and defaults are provided for its
echo     controls and parameters.
echo.
echo 3)  From this point the indexing run (for the target program selected)
echo     will proceed automatically, leaving you looking at its short output file
echo     in an editor of your choice (default = WordPad [Windows], EDIT [DOS]).
echo.
echo 4)  When you have finished with this, exit (Alt-F4 [WordPad], Alt-F,X [EDIT]).
echo     You will then be shown the updated solutions-summary file for this dataset,
echo     sorted into descending order of first I20 (indexed lines in the first 20),
echo     then figure of merit.  Again type Alt-F4 [Alt-F,X] to exit.
echo.
pause
echo 5)  The system version of Ton Spek's LePage program will then run, generating
echo     a detailed results file which includes the Niggli reduced cell for each
echo     solution, with corresponding sub-cell and super-cell analyses.  This is not
echo     displayed automatically as it can be bulky and requires experience to
echo     interpret, so the run proceeds directly to the revised summary in step 6.
echo.
echo 6)  A revised summary file will be displayed, listing the various solutions
echo     sorted on their reduced cell properties.
echo     Type Alt-F4 [Alt-F,X] to exit and return to Crysfire's command interface.
echo.
echo 7)  This will be a fresh start, so, if you wish to continue, any changes to general
echo     settings (such as Qmax) will need to be remade, although the current dataset
echo     should be reloaded for you automatically.
echo     If not, type QU (quit) to exit to the operating system.
echo.
pause
echo ** If CRYSFIRE fails to run, reporting "out of environment space", you have
echo    probably already used up most of your environment allocation, and so
echo    need to increase this to e.g. 1024 or 2048 bytes, using the SHELL
echo    command in CONFIG.SYS (/e parameter - e.g. /e:2048 for 2048 bytes)
echo.
echo    Thus to provide 2048 bytes of environment space, insert the one of the
echo    following "shell=" lines in your CONFIG.SYS file, as appropriate:
echo.
echo    (Windows 9x)       shell=C:\WINDOWS\COMMAND.COM C:\WINDOWS /e:2048 /p
echo    (DOS/Windows 3.x)  shell=C:\COMMAND.COM C:\DOS /e:2048 /p
echo.
echo    Special action is required under Windows ME (which ignores CONFIG.SYS):
echo    Right-click on the Crysfire (or MSDOS) icon on the Windows desktop,
echo    and choose "Properties" then "Memory".  Change the setting for the
echo    "Initial Environment" from "Auto" to a suitable value such as 2048
echo    (the largest available is 4096).
echo.
echo    Windows/XP and other NT-family O/S seem to manage without adjustment
echo.
echo For more information on CRYSFIRE and its associated scripts and programs,
echo see the user's manual for the system in CRYSFIRE.RTF (rich text format).
goto endbat
:exit
echo End of CRYSFIRE run (%_CFbitmode%-bit version)
:endbat
echo.
set _Csrt=
set _Csrtst=
set _C2Qdset=
set _C2Qid=
set _CFhome=
set _Q2id_ver=
